#!/usr/bin/env python3

import urllib.request
from lxml import etree
import tempfile, os, sys, glob, subprocess, shutil

class Build(object):
    LANGSTD = [ 'ar', 'de', 'en-GB', 'es', 'fr', 'it', 'ja', 'ko', 'pt', 'pt-BR', 'ru', 'zh-CN', 'zh-TW' ]

    def __init__(self, query, arch, url):
        """Build all versions that can be found in the indicated repo."""
        self.__query__ = query
        self.__arch__ = arch
        self.__url__ = url

        # Creating a tempfile
        self.__builddir__ = tempfile.mkdtemp()
        self.__tarballs__ = []
        self.__appname__ = ''
        self.__version__ = ''

        if self.__url__ == '-':
            print("Cannot find this version for arch {arch}.".format(arch = self.__arch__))
            return False


    def download(self):
        """Downloads the contents of the URL as it was a folder."""

        # Let's start with defining which files are to be downloaded.
        # Let's explore the remote folder.
        contents = etree.HTML(urllib.request.urlopen(self.__url__).read()).xpath("//td/a")
        self.__tarballs__ = [ x.text for x in contents if x.text.endswith('tar.gz') ]
        maintarball = self.__tarballs__[0]
        main_arr = maintarball.split('_')
        self.__appname__ = main_arr[0]
        self.__version__ = main_arr[1]

        os.chdir(self.__builddir__)
        for archive in self.__tarballs__:
            # Download the archive
            try:
                urllib.request.urlretrieve(self.__url__ + archive, archive)
            except:
                print("Failed to download {archive}.".format(archive = archive))

            print("Got %s." % archive)


    def build(self, outdir):
        """Building all the versions."""
        # We have 4 builds to do:
        # * standard languages, no help
        # * standard languages + offline help
        # * all languages, no help
        # * all languages + offline help

        # Preparation tasks
        appnamedir = os.path.join(self.__builddir__, self.__appname__)
        appimagedir = os.path.join(self.__builddir__, self.__appname__, self.__appname__ + '.AppDir')
        os.makedirs(appimagedir, exist_ok = True)
        # And then cd to the appname folder.
        os.chdir(appnamedir)
        # Download appimagetool from github
        appimagetoolurl = "https://github.com/AppImage/AppImageKit/releases/download/continuous/appimagetool-{arch}.AppImage".format(arch = self.__arch__)
        urllib.request.urlretrieve(appimagetoolurl, 'appimagetool')
        os.chmod('appimagetool', 0o755)

        # Run to build standard no help
        self.__unpackbuild__('standard', False)

        # Run to build standard full help
        self.__unpackbuild__('standard', True)

        # Build full no help
        self.__unpackbuild__('full', False)

        # Full with help
        self.__unpackbuild__('full', True)

    def __unpackbuild__(self, languageset = 'full', offlinehelp = False):
        # We start by filtering out tarballs from the list
        buildtarballs = [ self.__tarballs__[0] ] 

        # Let's process standard languages and append results to the
        # buildtarball
        if languageset == 'standard':
            for lang in Build.LANGSTD:
                buildtarballs.extend([ x for x in self.__tarballs__ if ('langpack_' + lang) in x ])
                if offlinehelp:
                    buildtarballs.extend([ x for x in self.__tarballs__ if ('helppack_' + lang) in x ])
        else:
            # In any other cases, we build with all languages
            if not offlinehelp:
                buildtarballs.extend([ x for x in self.__tarballs__ if 'langpack_' in x ])
            else:
                # We need also all help. Let's replace buildtarball with the
                # whole bunch
                buildtarballs = self.__tarballs__
                
        # Unpacking the tarballs
        for archive in buildtarballs:
            subprocess.run("tar xzf ../%s" % archive, shell=True)

        os.chdir(appimagedir)
        # At this point, let's decompress the deb packages
        subprocess.run("find .. -iname '*.deb' -exec dpkg -x {} . \;", shell=True)
        # Changing desktop file
        subprocess.run("find . -iname startcenter.desktop -exec cp {} . \;", shell=True)

        subprocess.run("find . -name '*startcenter.png' -path '*hicolor*48x48*' -exec cp {} . \;", shell=True)

        # Find the name of the binary called in the desktop file.
        binaryname = subprocess.check_output("awk 'BEGIN { FS = \"=\" } /^Exec/ { print $2; exit }' startcenter.desktop | awk '{ print $1 }'", shell=True).decode('utf-8').strip('\n')

        bindir=os.path.join(self.__builddir__, self.__appname__, self.__appname__ + '.AppDir', 'usr', 'bin')
        os.makedirs(bindir, exist_ok = True)
        os.chdir(bindir)
        subprocess.run("find ../../opt -name soffice -path '*programm*' -exec ln -s {} %s \;" % binaryname, shell=True)
        os.chdir(appimagedir)

        # Download AppRun from github
        apprunurl = "https://github.com/AppImage/AppImageKit/releases/download/continuous/AppRun-{arch}".format(arch = self.__arch__)
        urllib.request.urlretrieve(apprunurl, 'AppRun')
        os.chmod('AppRun', 0o755)
        
        # Setting app version
        appversion = self.__version__ + '.' + languageset
        if offlinehelp:
            appversion += '.help'

        # Building app
        subprocess.run("VERSION={version} ./appimagetool -v ./{appname}.AppDir/".format(version = appversion, appname = self.__appname__), shell=True)

        # Copying built image to final directory
        subprocess.run("find . -iname '*.AppImage' -exec cp {} %s \;" % outdir, shell = True)

        # Cleanup phase, before new run.
        os.chdir(appnamedir)
        for deb in glob.glob(appnamedir + '/*.deb'):
            os.remove(deb)
        shutil.rmtree(appimagedir)
        os.makedirs(appimagedir)


    def __del__(self):
        """Destructor"""
        #del self.__builddir__


if __name__ == '__main__':
    # Run if it is run as a program.
    # 1 -> query
    # 2 -> arch
    # 3 -> url
    # 4 -> outdir

    if not len(sys.argv) == 5:
        print("Please launch with this parameters: build.py query arch url outputdir")
        sys.exit(255)

    b = Build(sys.argv[1], sys.argv[2], sys.argv[3])
    b.download()
    b.build(sys.argv[4])
    del b
